# map script
## Fig V: Global maps of forecast biases (1) first month, (2) first season, (3) first year
## Fig VI: Global ACC maps in the (1) first season/year, (2) 2-5 years, (3) 6-10 years

#Clear workspace
rm(list = ls())

library(s2dv)
library(lubridate)
library(easyNCDF)
library(startR)

# TRUE: run the script on Nord4, serial computing
# FALSE: run the script on WS and dispatch the jobs to nord4, parallel computing
RUN_LOCAL <- FALSE
retrieve <- FALSE
save_plot_to <- '/esarchive/scratch/aho/cp_shiny_plots/'

exp <- 'a3w5'
vari <- 'psl' #'tas'
mod <- 'Amon'
grid <- 'gr' #'1x1'
#NOTE: era5_tas from 197901 to 202006; era5_psl start from 198001 to 201912
yr1 <- '1980' #'1979'
yr2 <- '2009'  #2020 - floor(t2/12)?
t1 <- 1
t2 <- 14 # for first whole year (3:14 = JAN to DEC) 
t2 <- 2 + (12 * 10) # fyear 1:10
chosen_season <- 'Annual' #'DJF'
lats <- c(-90, 90)
lons <- c(0, 359.9)

# original_files
membs <- c('r1i4p1f1','r2i4p1f1','r3i4p1f1','r4i4p1f1','r5i4p1f1','r6i4p1f1','r7i4p1f1','r8i4p1f1','r9i4p1f1','r10i4p1f1')
# diags
#membs <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1','r7i1p1f1','r8i1p1f1','r9i1p1f1','r10i1p1f1')

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------
# diags
#repos <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/',
#                mod, '/$var$/', grid, '/v*/$var$_', mod,
#                '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid,
#                '_$chunk$.nc')

# original_files
repos <- paste0('/esarchive/exp/ecearth/', exp, '/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/', mod, '/$var$/', grid, '/v*/$var$_', mod,
                '_EC-Earth3_dcppA-hindcast_s$sdate$-$memb$_', grid,
                '_$chunk$.nc')
# [lat = 256 ; lon = 512]

data <- Start(dat = repos,
              var = vari,
              memb = membs,
              sdate = paste(seq(yr1, yr2)),
              chunk = 'all',
              chunk_depends = 'sdate',
              time = c(t1:t2), #'all',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
#              latitude = 'all',
              latitude_reorder = Sort(),
#              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
#              transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = 'r360x180', method = 'con', crop = FALSE), transform_vars = c('lat','lon'),
              return_vars = list(time = c('sdate', 'chunk'),
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)

attr(data, 'Dimensions')

#--------------------------
# Step2: Load obs
#--------------------------
obs <- 'era5'
path_obs <- paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/$var$_f1h-r512x256/$var$_$file_date$.nc')
time_data <- attr(data, 'Variables')$common$time #[sdate, time]
dim(time_data)

#---------------------------------------------
##Change the version before bug being fixed.
#detach("package:startR", unload = T)
#require(startR, lib.loc = "/shared/earth/software/R/3.6.1-foss-2015a-bare/lib64/R/library") #change the loaded library
#---------------------------------------------


#-------------NOTE WORKING NOW----------------------------------------
# Check obs time value first to avoid date difference
first_obs_file <- paste0(paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/', vari, '_f1h-r512x256/'),
                         list.files(paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/', vari, '_f1h-r512x256/'))[1])

tmp <- NcOpen(first_obs_file)
obs_time_1 <- NcToArray(tmp, vars_to_read = 'time',dim_indices = list(time = 1))
NcClose(tmp)
obs_time_unit <- attr(obs_time_1, "variables")$time$units
tmp <- strsplit(obs_time_unit, " ")[[1]]
origin <- suppressWarnings(paste(tmp[which(!is.na(as.numeric(sapply(tmp, substr, 1, 1))))], collapse = ' '))
if (tmp[1] %in% c("hours", "hour", "days", "day")) {
  if (tmp[1] %in% c("hours", "hour")) {
    obs_time_1 <- obs_time_1 * 60 * 60
  } else if (tmp[1] %in% c("days", "day")) {
    obs_time_1 <- obs_time_1 * 60 * 60 * 24
  }
}
obs_day <- day(as.POSIXct(obs_time_1, tz = 'UTC', origin = origin))
data_day <- day(time_data[1])
if (obs_day != data_day) {
  # Shift time_data to obs_day
#  time_data <- as.POSIXct(array(time_data + days(obs_day - data_day), dim = dim(time_data)),
#                          origin = '1970-01-01', tz = 'UTC')

  time_data <- as.POSIXct(array(floor_date(time_data, unit = 'month'), dim = dim(time_data)),
                          origin = '1970-01-01', tz = 'UTC')

}
#-----------------------------------------------------
#NOTE: This Start call may have time shift problem.
data_obs <- Start(dat = path_obs,
              var = vari,
              file_date = unique(format(time_data, '%Y%m')),
              time = values(time_data),
              time_across = 'file_date',
              merge_across_dims = TRUE,
              split_multiselected_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
#              transform = CDORemapper, transform_extra_cells = 2,
#              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',
#                                      method = 'con'),
#              transform_vars = c('latitude', 'longitude'),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('file_date'),
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)

#NOTE: If obs data only have one time step, then use this Start call.
file_date_array <- array(format(time_data, '%Y%m'), dim = c(sdate = 30, time = 122))
data_obs <- Start(dat = path_obs,
              var = vari,
              file_date = file_date_array, 
              split_multiselected_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
#              transform = CDORemapper, transform_extra_cells = 2,
#              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',
#                                      method = 'con'),
#              transform_vars = c('latitude', 'longitude'),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('file_date'),
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)


attr(data_obs, 'Dimensions')

#============== CHECK ATTRIBUTES ==================
## dimensions
dim_data <- attr(data, 'Dimensions')
dim_obs <- attr(data_obs, 'Dimensions')
if (any(!names(dim_obs) %in% names(dim_data))) {
  stop("obs does not have the same dimension names as exp.")
} else if (any(dim_data[match(names(dim_obs), names(dim_data))] != dim_obs)) {
  stop("obs and exp do not have the same dimensions.")
}

## time
time_obs <- attr(data_obs, 'Variables')$common$time 
if (!all.equal(format(time_obs, '%Y%m'), format(time_data, '%Y%m'))) {
  stop("obs and exp do not have the same time (%Y%m).")
}

## lat and lon
lat_obs <- attr(data_obs, 'Variables')$common$latitude
lon_obs <- attr(data_obs, 'Variables')$common$longitude
lat_data <- attr(data, 'Variables')$common$latitude
lon_data <- attr(data, 'Variables')$common$longitude
if (!all.equal(lat_obs, lat_data , check.attributes = F) |
    !all.equal(lon_obs, lon_data, check.attributes = F)) {
  stop("obs and exp do not have the same lat/lon.")
}
#=====================================================


#--------------------------
# Step3: Build workflow
#--------------------------
  source('~/aho-testtest/shiny/cp_shiny_decadal/fun_Season_diffClim_ACC.R')

  step <- Step(fun = fun_Season_diffClim_ACC,
               use_libraries = c('s2dv', 'ClimProjDiags'),
               target_dims = list(data = c('sdate', 'time', 'memb'),
                                  data_obs = c('sdate', 'time')),
               output_dims = list(res_bias = 'time',
                                  res_acc = 'time',
                                  res_acc_sig = 'time'))


#  if (chosen_season == 'Annual') {
#    chosen_mon <- c(1, 12)
#  } else if (chosen_season == 'DJF') {
#    chosen_mon <- c(12, 2)
#  } else if (chosen_season == 'JJA') {
#    chosen_mon <- c(6, 8)
#  } else if (chosen_season == 'SON') {
#    chosen_mon <- c(9, 11)
#  } else {
#    stop("Undefined season. Assign another chosen_season.")
#  }

  wf <- AddStep(inputs = list(data, data_obs),
                step_fun = step)#, chosen_mon = chosen_mon)


#--------------------------
# Step4: Compute
#--------------------------
if (RUN_LOCAL) {
# local
  output <- Compute(workflow = wf$res_bias, #diff,
                    threads_load = 2, threads_compute = 4,
                    chunks = list(longitude = 6))
dim(output$diff)

} else {
# Nord3v2
#-------------------user-defined---------------------
  queue_host <- 'nord4'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------
  output <- Compute(wf$res_bias,
                 chunks = list(latitude = 10,
                               longitude = 10),#$output1
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(
                   queue_host = queue_host,
                   queue_type = 'slurm',
                   temp_dir = temp_dir,
                   cores_per_job = 16,
                   job_wallclock = '01:00:00',
                   max_jobs = 16,
                   bidirectional = FALSE,
                   polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = FALSE
                 )  #$output1
}
saveRDS(output, paste0('~/aho-testtest/shiny/cp_shiny_decadal/mapdata_', vari, '_256x512_',
                       paste0(yr1,'-',yr2), '.rds'))

tmp <- readRDS(paste0('~/aho-testtest/shiny/cp_shiny_decadal/mapdata_', vari, '_256x512_', paste0(yr1,'-',yr2), '.rds'))
output <- Collect(tmp, wait = TRUE)

#--------------------------
# Step5: Plotting
#--------------------------

output <- readRDS('~/aho-testtest/shiny/cp_shiny_decadal/mapdata.rds')

bias <- drop(output$res_bias)
acc_data <- drop(output$res_acc)
acc_sig <- drop(output$res_acc_sig)

lon <- as.vector(attr(data, 'Variables')$common$longitude)
lat <- as.vector(attr(data, 'Variables')$common$latitude)

color_range_bias <- c(-max(abs(range(bias))), max(abs(range(bias))))
color_range_bias <- c(-5, 5)

# Period of each "time" dim
period_bias <- c('Mon1', 'DJF1', 'MAM1', 'JJA1', 'SON1', 'Yr1')
period_acc <- c('DJF1', 'MAM1', 'JJA1', 'SON1', 'Yr1', 'Yr2-5', 'Yr6-10')

# diff
for (time_i in 1:dim(bias)[1]) {
  PlotEquiMap(bias[time_i, , ], lon = lon, lat = lat,
              toptitle = paste0('bias (to ', obs, ') ', exp, ', ', 
                           vari, ', ', period_bias[time_i], ', ',
                           paste0(yr1,'-',yr2)),
              filled.continents = FALSE, bar_limits = color_range_bias, 
              fileout = paste0(save_plot_to, 'dcpp_map_bias_', vari, '_', 
                               period_bias[time_i], '_', paste0(yr1,'-',yr2), '.png'))
}

# acc
for (time_i in 1:dim(acc_data)[1]) {
  PlotEquiMap(acc_data[time_i, , ], lon = lon, lat = lat,
              toptitle = paste0('ACC (to ', obs, ') ', exp, ', ', 
                           vari, ', ', period_acc[time_i], ', ',
                           paste0(yr1,'-',yr2)),
              filled.continents = FALSE, bar_limits = c(-1, 1), 
              dots = acc_sig[time_i, , ],
              fileout = paste0(save_plot_to, 'dcpp_map_acc_', vari, '_',
                               period_acc[time_i], '_', paste0(yr1,'-',yr2), '.png'))
}

