#Clear workspace
rm(list = ls())

#library(RColorBrewer)
library(s2dv)
library(ncdf4)
library(startR)
library(psych)

DEBUG <- TRUE
savefile <- TRUE
savefile_path <- '/esarchive/scratch/aho/cp_shiny_plots/'
INFO <- TRUE
Write_to_log <- TRUE
# logger
if (Write_to_log) {
file('/esarchive/scratch/aho/cp_shiny_plots/log_timeseries.txt', open = 'a')
sink('/esarchive/scratch/aho/cp_shiny_plots/log_timeseries.txt')
print('============START TIMESERIES SCRIPT================')
}

# Insert variables:
exp <- 'a4ai' #'a3w5'
recon <- 'a3t4'  # for Ocean
vari <- 'tosmean'
dcpp <- 'dcppA-hindcast'
mod <- 'Omon'
grid <- 'gn'
#QUESTION: all the members? If so, automatically search and get from path; if not, member is a variable
if (exp == 'a4ai') {
  membs_exp <- c('r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1') #, 'r7i1p1f1', 'r8i1p1f1', 'r9i1p1f1', 'r10i1p1f1', 'r11i1p1f1')
} else if (exp == 'a3w5') {
membs_exp <- c('r1i4p1f1','r2i4p1f1','r3i4p1f1','r4i4p1f1','r5i4p1f1','r6i4p1f1','r7i4p1f1','r8i4p1f1','r9i4p1f1','r10i4p1f1') 
}
membs_recon <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1')

#QUESTION: all the years? If so, automatically search and get from path; if not, yr1/yr2 is a variable
yr1 <- '1960'
yr2 <- '2020'
reg <- 'Nino3.4' #'Global_Ocean'

#QUESTION: all the time steps? If so, open the first file to get the time step; if not, t1/y2 is a variable
#t1 <- 1
#t2 <- 132

if (INFO) {
print(paste0("Start loading data: pred = ", exp, ", recon = ", recon, ", vari = ", vari,
      ", reg = ", reg))
}

#===== STEP1: Load CMIP6 DCPP =====

# Variable preparation
# sdate, path
##-----1. If always want the whole period:
#EC-Earth3 is changable (*) only one
first_file_path_till_ver <- paste0(Sys.glob(paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/*/')), dcpp, '/', membs_exp[1], '/',
                                   mod, '/', vari, '/', grid)

#QUESTION: two versions here in a3w5
ver <- list.files(first_file_path_till_ver)
#if (exp == 'a4ai') {
#  files <- list.files(paste0(first_file_path_till_ver, '/', ver[1], '/'))
#} else if (exp == 'a3w5') {
#  files <- list.files(paste0(first_file_path_till_ver, '/', ver[2], '/'))  # ver[2] for a3w5
#}
files <- list.files(paste0(first_file_path_till_ver, '/', ver[1], '/'))
sdates_memb <- unique(sapply(lapply(lapply(files, strsplit, '_'), '[[', 1), '[', 5))
sdates <- sapply(lapply(lapply(sdates_memb, strsplit, '-'), '[[', 1), '[', 1)  #s19601101
if (DEBUG) {
  if (any(substr(sdates, 1, 1) != 's') | any(sapply(sdates, nchar) != 5) |
      any(is.na(as.integer(substr(sdates, 2, 5))))) {
    stop("Incorrect sdate format.")
  }
}
path <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/*/dcppA-hindcast/$memb$/', mod, '/$var$/',
               grid, '/v*/$var$_', mod, '_*_dcppA-hindcast_$sdate$-$memb$_', grid, '_$chunk$.nc')

##-----2. If the period is selected:
#path <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/', mod, '/$var$/',
#               grid, '/v*/$var$_', mod, '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid, '_$chunk$.nc')
#sdates <- paste(seq(yr1, yr2, by = int))

# time
##-----1. If always want the maximal time steps:
# Need to open the first file and get the inner dimensions

if (exp %in% c('a3w5', 'a4ai') & vari %in% c('tosmean') & reg %in% c('Nino3.4', 'Global_Ocean')) {
data <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
                       exp, '_', vari, '_', reg, '.rds'))
data_dates <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
                       exp, '_', vari, '_', reg, '_dates.rds')) 

} else {
# Load data
data <- Start(dat = path,
              var = vari,
              memb = membs_exp,
              region = c(reg),
              sdate = sdates,
              time = 'all', #c(t1:t2),
              chunk = 'all',
              chunk_depends = 'sdate',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              return_vars = list(time = c('sdate', 'chunk'), region = NULL),
              num_procs = 8,
              retrieve = T)

data <- drop(data)
data_dates <- attr(data, 'Variables')$common$time
}

if (INFO) {
  print("dim(data):")
  print(dim(data))
  print("dim(data_dates):")
  print(dim(data_dates))
}

if (DEBUG) {
  if (any(names(dim(data)) != c('memb', 'sdate', 'time'))) {
    stop("Unexpected data dimensions.")
  }
  if (any(names(dim(data_dates)) != c('sdate', 'time'))) {
    stop("Unexpected data time attribute (data_dates) dimensions.")
  }
}
if (INFO) print("==========STEP1 FINISHED.===========")

#===== STEP2: Load reconstruction =====
# Variable preparation

if (recon %in% c('a3t4') & vari %in% c('tosmean') & reg %in% c('Nino3.4', 'Global_Ocean')) {
data_recon <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
                       recon, '_', vari, '_', reg, '.rds'))
recon_dates <- readRDS(paste0('/esarchive/scratch/aho/cp_shiny_plots/data_rds/data_',
                       recon, '_', vari, '_', reg, '_dates.rds'))

} else {
# path
path <- paste0('/esarchive/exp/ecearth/', recon, '/diags/CMIP/EC-Earth-Consortium/EC-Earth3-LR/piControl/$memb$/' , mod, '/$var$/',
               grid, '/v*/$var$_', mod, '_EC-Earth3-LR_piControl_$memb$_', grid, '_$chunk$.nc')

data_recon <- Start(dat = path,
              var = vari,
              memb = membs_recon,
              region = c(reg),
              time = 'all',
              chunk = 'all',
#              chunk_depends = 'sdate',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              return_vars = list(time = 'chunk', region = NULL),
              num_procs = 8,
              retrieve = T)
recon_dates <- attr(data_recon, 'Variables')$common$time
data_recon <- InsertDim(drop(data_recon), 2, 1, 'sdate')
}

if (INFO) {
  print("dim(data_recon):")
  print(dim(data_recon))
  print("dim(recon_dates):")
  print(dim(recon_dates))
}

if (DEBUG) {
  if (any(names(dim(data_recon)) != c('memb', 'sdate', 'time'))) {
    stop("Unexpected data_recon dimensions.")
  }
}

#Hist to hind
#QUESTION: why until 2021? exp is till 2020, and sdate = 62 is empty 
#recon_sdates <- gsub("-", "", paste(seq(as.Date("1960/11/01"), as.Date("2021/11/01"), "years"), sep = "-"))

#sdates_out <- strftime(seq(as.Date(data_dates[1, 1]), as.Date(data_dates[dim(data_dates)['sdate'], 1]), "years"), "%Y%m")
#data_recon_hc <- Reorder(Histo2Hindcast(data_recon, sdatesin = strftime(recon_dates[1], "%Y%m"), sdatesout = sdates_out, nleadtimesout = dim(data)['time'], sdate_dim = "sdate", ftime_dim = "time"),
#                         c('memb', 'sdate', 'time'))

sdates_out <- strftime(seq(as.Date("1960-11-01"), as.Date("2020-11-01"), "years"), "%Y%m")
data_recon_hc <- Reorder(Histo2Hindcast(data_recon, 
                           sdatesin = strftime(recon_dates[1], "%Y%m"),
                           sdatesout = sdates_out,
                           nleadtimesout = dim(data)['time'], 
                           sdate_dim = "sdate", ftime_dim = "time"),
                         c('memb', 'sdate', 'time'))


if (INFO) {
print("dim(data_recon_hc):")
print(dim(data_recon_hc))
}

#NOTE: data_recon_hc is the subset from the first to last year of data. Even if data is discontinuous, data_recon_hc is continuous.

if (DEBUG) {
  if (any(names(dim(data_recon_hc)) != c('memb', 'sdate', 'time'))) {
    stop("Unexpected data_recon_hc dimensions.")
  }
  if (any(dim(data)['time'] != dim(data_recon_hc)['time'])) {
    stop("The time dimension in data and data_recon_hc is not the same.")
  }
}
if (INFO) print("==========STEP2 FINISHED.===========")

#===== STEP3: Load EC-Earth historical runs =====
#QUESTION: how to store the pre-processing data? And the file name regulation to get the file automatically? Should the data be generated automatically?
nc_file <- paste0('/esarchive/scratch/Earth/rfernand/monitor_dcpp/hist_', vari, '_', reg, '_s1960-s2022.nc')
nc_data <- nc_open(nc_file)
#nc_data <- nc_open('/esarchive/scratch/Earth/rfernand/monitor_dcpp/hist_tosmean_Global_Ocean_s1960-s2022.nc')
data_hist <- ncvar_get(nc_data, vari)
nc_close(nc_data)
names(dim(data_hist)) <- c("memb", "sdate", "time")
if (INFO) {
print("dim(data_hist):")
print(dim(data_hist))
}

if (INFO) print("==========STEP3 FINISHED.===========")

#===== STEP4: Load HadISST =====
#QUESTION: same as above
obs <- "hadisst"
nc_file <- paste0('/esarchive/scratch/Earth/rfernand/monitor_dcpp/', obs, '_', vari, '_', reg, '_s1960-s2020.nc')
nc_data <- nc_open(nc_file)
#nc_data <- nc_open('/esarchive/scratch/Earth/rfernand/monitor_dcpp/hadisst_tosmean_Global_Ocean_s1960-s2020.nc')
#QUESTION: The unit transfer and dim adjustment should be done when generating the nc file
data_obs <- InsertDim(ncvar_get(nc_data, "tos"), 1, 1, 'memb') - 273.15
nc_close(nc_data)
names(dim(data_obs)) <- c("memb", "sdate", "time")
if (INFO) {
print("dim(data_obs)")
print(dim(data_obs))
}

if (INFO) print("==========STEP4 FINISHED.===========")

#===== STEP5: Compute annual means ===== (only global mean)
#Question: Which variables need to compute annual mean? Which use monthly data directly?
if (reg %in% c('Global_Ocean')) {
    
  #exp
  data_yr <- Reorder(Season(data, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE),
                     c(2, 3, 1))
  dim(data_yr)
  if (DEBUG) {
    if (any(names(dim(data_yr)) != c('memb', 'sdate', 'time'))) {
      stop("Unexpected data_yr dimensions.")
    }
  }
  
  #recon
  data_recon_yr <- Reorder(Season(data_recon_hc, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE),
                           c(2, 3, 1))
  dim(data_recon_yr)
  if (DEBUG) {
    if (any(names(dim(data_recon_yr)) != c('memb', 'sdate', 'time'))) {
      stop("Unexpected data_recon_yr dimensions.")
    }
  }
  
  #Historical
  data_hist_yr <- Reorder(Season(data_hist, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE), c(2, 3, 1))
  dim(data_hist_yr)
  if (DEBUG) {
    if (any(names(dim(data_hist_yr)) != c('memb', 'sdate', 'time'))) {
      stop("Unexpected data_hist_yr dimensions.")
    }
  }
  
  #HadISST
  data_obs_yr <- Reorder(Season(data_obs, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE), c(2, 3, 1))
  dim(data_obs_yr)
  dim(data_hist_yr)
  if (DEBUG) {
    if (any(names(dim(data_obs_yr)) != c('memb', 'sdate', 'time'))) {
      stop("Unexpected data_obs_yr dimensions.")
    }
  }
}

if (INFO) print("==========STEP5 FINISHED.===========")


# t2 is the xth year since 1960, the start year of recon. 1960 is 1st yr, 1961 is 2nd yr, etc.
t2 <- as.numeric(substr(data_dates[, 1], 1, 4)) - as.numeric(substr(recon_dates[1], 1, 4)) + 1


#===== Plotting: Timeseries based on monthly means (type 1a) =====
if (savefile) {
  png(paste0(savefile_path, '/dcpp_ts_figIa_', exp, '_', vari, '_', reg, '.png'),
      width = 480*1.8, height = 480*0.8)
} else {
  dev.new(width = 9, height = 5)
}
#PLOT
par(mfrow = c(2, 1), cex = 0.65)
pcexlab <- 1.3
pcexmain <- 1.3
pcexax <- 1.3

brks_def <- c(seq(1, dim(data)['sdate'], by = 4))
colors <- readRDS('/esarchive/scratch/Earth/rfernand/dcpp/colorbar_21.RDS')
col_fun <- colorRampPalette(colors[1:20])
jBrewColorsano <- c(col_fun(length(brks_def) - 1))

nf <- 6 # first 6 ftime
max_range <- range(range(data[, , 1:nf]), range(data_recon))
enlarge_ratio <- 0.00
ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)

# 1st part
# create the frame and draw the first member
plot(c(1:732), data_recon[1, , 1:732], col = "grey", type = "l", lwd = 0.5, ylim = ylim, xlim = c(1, 350), cex.lab = pcexlab, cex.main = pcexmain, cex.axis = pcexax,
     xlab = "Time", ylab = "T", main = paste0('a) ', reg, ', ', vari, ', ', exp), lty = 1, xaxt = "n")
axis(1, at = seq(1, 732, by = 12), labels = paste0(seq(as.numeric(substr(recon_dates[1], 1, 4)), as.numeric(substr(recon_dates[length(recon_dates)], 1, 4)))))
     
# draw the rest of member
for (i in seq(2, dim(data_recon)['memb'])) {
  lines(c(1:732), data_recon[i, , ], col = "grey", type = "l", lwd = 0.5)
}

# draw exp, memb by memb, sdate by sdate
for (i in seq(1, dim(data)['memb'])) {
  for (t in seq(1, length(t2))) {
    # the chosen ftime (1:nf) + months since 1960. The ini month is 11.
    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = 'blue', type = "l", lwd = 0.5) 
#    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = jBrewColorsano[t / 4.5 + 1], type = "l", lwd = 0.5)
  }
}
# add legend
legend(1, ylim[2], legend = c(exp, recon), col = c("blue", 'grey'), lty = 1, bty = "n")


# 2nd part
plot(c(1:732), data_recon[1, , 1:732], col = "grey", type = "l", lwd = 0.5, ylim = ylim, xlim = c(351,732), cex.lab = pcexlab, cex.main = pcexmain, cex.axis = pcexax,
     xlab = "Time", ylab = "T", main = paste0('a) ', reg, ', ', vari, ', ', exp, ' (cont.)'), lty = 1, xaxt = "n")
axis(1, at = seq(1, 732, by = 12), labels = paste0(seq(as.numeric(substr(recon_dates[1], 1, 4)), as.numeric(substr(recon_dates[length(recon_dates)], 1, 4)))))

# draw the rest of member
for (i in seq(2, dim(data_recon)['memb'])) {
  lines(c(1:732), data_recon[i, , ], col = "grey", type = "l", lwd = 0.5)
}

# draw exp, memb by memb, sdate by sdate
for (i in seq(1, dim(data)['memb'])) {
  for (t in seq(1, length(t2))) {
    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = 'blue', type = "l", lwd = 0.5) 
#    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = jBrewColorsano[t / 4.5 + 1], type = "l", lwd = 0.5)
  }
}

if (savefile) dev.off()

if (INFO) print("==========PLOTTING TYPE I FINISHED.===========")


#===== STEP6: Compute Clim and Ano =====

#Extract the same number of startdates and member

#QUESTION: why 60? Can it be 61 (the shortest sdate)?
##Not used
sds <- 60 # min(sapply(list(dim(data_yr), dim(data_recon_yr), dim(data_hist_yr), dim(data_obs_yr)), '[', 'sdate')) # --> it doesn't work for discontinuous data (a4ai)

# If Nino3.4, the first 30 months; if Global_Means, all the years
monf <- ifelse(reg %in% c('Nino3.4'), 30, dim(data_yr)['time'])

if (reg %in% c('Nino3.4')) {
    data_tmp <- data
    recon_tmp <- data_recon_hc
    hist_tmp <- data_hist
    obs_tmp <- data_obs
} else {
    data_tmp <- data_yr
    recon_tmp <- data_recon_yr
    hist_tmp <- data_hist_yr
    obs_tmp <- data_obs_yr
}

# Subset to the same time and sdate; hist has the same member as data
data_ss <- ClimProjDiags::Subset(data_tmp, c('time'), list(1:monf), drop = FALSE)
#recon_ss <- ClimProjDiags::Subset(recon_tmp, c('sdate', 'time'), list(t2 - t2[1] + 1, 1:monf), drop = FALSE)
recon_ss <- ClimProjDiags::Subset(recon_tmp, c('sdate', 'time'), list(t2, 1:monf), drop = FALSE)
hist_ss <- ClimProjDiags::Subset(hist_tmp, c('memb', 'sdate', 'time'), list(1:min(dim(hist_tmp)['memb'], dim(data_tmp)['memb']), t2, 1:monf), drop = FALSE)
obs_ss <- ClimProjDiags::Subset(obs_tmp, c('sdate', 'time'), list(t2, 1:monf), drop = FALSE)

data_ss <- InsertDim(data_ss, 1, 1, 'dat')
recon_ss <- InsertDim(recon_ss, 1, 1, 'dat')
hist_ss <- InsertDim(hist_ss, 1, 1, 'dat')
obs_ss <- InsertDim(obs_ss, 1, 1, 'dat')

# hist need to have a full sdate and member line, and obs and recon need to have a full sdate to compare with full hist.
hist_ss_full <- ClimProjDiags::Subset(hist_tmp, c('sdate', 'time'),
                       list(1:dim(obs_tmp)['sdate'], 1:monf), drop = FALSE)
obs_ss_full <- ClimProjDiags::Subset(obs_tmp, c('time'), list(1:monf), drop = FALSE)
recon_ss_full <- ClimProjDiags::Subset(recon_tmp, c('sdate', 'time'), list(1:dim(obs_tmp)['sdate'], 1:monf), drop = FALSE)
hist_ss_full <- InsertDim(hist_ss_full, 1, 1, 'dat')
obs_ss_full <- InsertDim(obs_ss_full, 1, 1, 'dat')
recon_ss_full <- InsertDim(recon_ss_full, 1, 1, 'dat')

# When the bug is fixed...
#data_yrs <- ClimProjDiags::Subset(data_yr, 'sdate', 1:sds, drop = FALSE)
#data_recon_yrs <- ClimProjDiags::Subset(data_recon_yr, 'sdate', 1:sds, drop = FALSE)
#data_hist_yrs <- ClimProjDiags::Subset(data_hist_yr, 'sdate', 1:sds, drop = FALSE)
#data_obs_yrs <- ClimProjDiags::Subset(data_obs_yr, 'sdate', 1:sds, drop = FALSE)
if (INFO) {
print("dim(data_ss):")
print(dim(data_ss))
print("dim(recon_ss):")
print(dim(recon_ss))
print("dim(hist_ss):")
print(dim(hist_ss))
print("dim(obs_ss):")
print(dim(obs_ss))

print("dim(hist_ss_full):")
print(dim(hist_ss_full))
print("dim(obs_ss_full):")
print(dim(obs_ss_full))
print("dim(recon_ss_full):")
print(dim(recon_ss_full))
}

if (DEBUG) {
  tmp_dimnames <- c('dat', 'memb', 'sdate', 'time')
  if (any(names(dim(data_ss)) != tmp_dimnames) | 
      any(names(dim(recon_ss)) != tmp_dimnames) |
      any(names(dim(hist_ss)) != tmp_dimnames) |
      any(names(dim(obs_ss)) != tmp_dimnames)) {
    stop("Unexpected xxx_ss dimensions. They're used as Clim input.")
  }
  if (!identical(
        dim(data_ss)[c('sdate', 'time')], 
        dim(recon_ss)[c('sdate', 'time')], 
        dim(hist_ss)[c('sdate', 'time')], 
        dim(obs_ss)[c('sdate', 'time')])) {
    stop("The data inputs for Clim should have the same sdate and time dimensions.")
  }
  # full hist and obs
  if (!identical(
        dim(hist_ss_full)[c('sdate', 'time')],
        dim(obs_ss_full)[c('sdate', 'time')],
        dim(recon_ss_full)[c('sdate', 'time')])) {
    stop("The data inputs for Clim should have the same sdate and time dimensions. hist_full and obs_full don't share the same dimensions.")
  }
}


# Calculate Clim
clim_data <- Clim(data_ss, obs_ss, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                  method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
#dim(clim_data$clim_exp)
# dat time 
#   1   30 
clim_recon <- Clim(recon_ss, obs_ss, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                   method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
clim_hist <- Clim(hist_ss, obs_ss, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                  method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
# Full hist period (all sdates)
clim_hist_full <- Clim(hist_ss_full, obs_ss_full, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                  method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
clim_recon_full <- Clim(recon_ss_full, obs_ss_full, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                  method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)

if (INFO) {
print("====Clim output===========")
print("dim(clim_data$clim_exp):")
print(dim(clim_data$clim_exp))
print("dim(clim_recon$clim_exp):")
print(dim(clim_recon$clim_exp))
print("dim(clim_data$clim_obs):")
print(dim(clim_data$clim_obs))
print("dim(clim_hist$clim_exp):")
print(dim(clim_hist$clim_exp))
print("dim(clim_hist_full):")
print(dim(clim_hist_full$clim_exp))
print("dim(clim_recon_full):")
print(dim(clim_recon_full$clim_exp))
}

# Calculate Anomalies & Ensemble mean
data_ss_anom <- Ano(data_ss, clim_data$clim_exp)
data_ss_anom_ens <- MeanDims(data_ss_anom, 'memb')

recon_ss_anom <- Ano(recon_ss, clim_recon$clim_exp)
recon_ss_anom_ens <- MeanDims(recon_ss_anom, 'memb')

obs_ss_anom <- Ano(obs_ss, clim_data$clim_obs)
obs_ss_anom_ens <- MeanDims(obs_ss_anom, 'memb')

hist_ss_anom <- Ano(hist_ss, clim_hist$clim_exp)
hist_ss_anom_ens <- MeanDims(hist_ss_anom, 'memb')

hist_ss_full_anom <- Ano(hist_ss_full, clim_hist_full$clim_exp)
hist_ss_full_anom_ens <- MeanDims(hist_ss_full_anom, 'memb')

obs_ss_full_anom <- Ano(obs_ss_full, clim_hist_full$clim_obs)
obs_ss_full_anom_ens <- MeanDims(obs_ss_full_anom, 'memb')

recon_ss_full_anom <- Ano(recon_ss_full, clim_recon_full$clim_exp)
recon_ss_full_anom_ens <- MeanDims(recon_ss_full_anom, 'memb')

if (INFO) {
print("====Ano+ens_mean output===========")
print("dim(data_ss_anom_ens):")
print(dim(data_ss_anom_ens))
print("dim(recon_ss_anom_ens):")
print(dim(recon_ss_anom_ens))
print("dim(obs_ss_anom_ens):")
print(dim(obs_ss_anom_ens))
print("dim(hist_ss_anom_ens):")
print(dim(hist_ss_anom_ens))
print("dim(hist_ss_full_anom_ens):")
print(dim(hist_ss_full_anom_ens))
print("dim(obs_ss_full_anom_ens):")
print(dim(obs_ss_full_anom_ens))
print("dim(recon_ss_full_anom_ens):")
print(dim(recon_ss_full_anom_ens))
}

if (DEBUG) {
  tmp_dimnames <- c('dat', 'sdate', 'time')
  if (any(names(dim(data_ss_anom_ens)) != tmp_dimnames) |
      any(names(dim(recon_ss_anom_ens)) != tmp_dimnames) |
      any(names(dim(hist_ss_anom_ens)) != tmp_dimnames) |
      any(names(dim(obs_ss_anom_ens)) != tmp_dimnames)) {
    stop("Unexpected xxx_ss_anom_ens dimensions.")
  }
  if (!identical(
        dim(data_ss_anom_ens)[c('sdate', 'time')],
        dim(recon_ss_anom_ens)[c('sdate', 'time')],
        dim(hist_ss_anom_ens)[c('sdate', 'time')],
        dim(obs_ss_anom_ens)[c('sdate', 'time')])) {
    stop("The data inputs for Clim should have the same sdate and time dimensions.")
  }
  if (!identical(
        dim(hist_ss_full_anom_ens)[c('sdate', 'time')],
        dim(obs_ss_full_anom_ens)[c('sdate', 'time')],
        dim(recon_ss_full_anom_ens)[c('sdate', 'time')])) {
    stop("The data inputs for Clim should have the same sdate and time dimensions. Not same for full hist and full obs.")
  }
}

if (INFO) print("==========STEP6 FINISHED.===========")

#===== STEP7: Compute Skill (ACC) =====
# figIV
acc_data <- Corr(data_ss_anom_ens, obs_ss_anom_ens, time_dim = 'sdate', dat_dim = 'dat', 
                 method = 'pearson', pval = TRUE, conf = F)

acc_hist <- Corr(hist_ss_anom_ens, obs_ss_anom_ens, time_dim = 'sdate', dat_dim = 'dat', 
                 method = 'pearson', pval = TRUE, conf = F)
acc_hist_full <- Corr(hist_ss_full_anom_ens, obs_ss_full_anom_ens, time_dim = 'sdate', dat_dim = 'dat',
                      method = 'pearson', pval = TRUE, conf = F)

# figIVb
acc_data_recon <- Corr(data_ss_anom_ens, recon_ss_anom_ens, time_dim = 'sdate', dat_dim = 'dat',
                       method = 'pearson', pval = TRUE, conf = F)
acc_hist_recon <- Corr(hist_ss_anom_ens, recon_ss_anom_ens, time_dim = 'sdate', dat_dim = 'dat',
                       method = 'pearson', pval = TRUE, conf = F)
acc_hist_recon_full <- Corr(hist_ss_full_anom_ens, recon_ss_full_anom_ens, time_dim = 'sdate', dat_dim = 'dat',
                       method = 'pearson', pval = TRUE, conf = F)

if (DEBUG) {
  if (!identical(
        dim(acc_data$corr)['time'],
        dim(acc_hist$corr)['time'],
        dim(acc_data_recon$corr)['time'],
        dim(acc_hist_recon$corr)['time'])) {
    stop("The ACC outputs should have the same time dimensions.")
  }
}


# ACC significance: Whether the correlation is significant
acc_data_sig <- as.integer(acc_data$p.val[1, , ] < 0.05)
acc_data_sig[acc_data_sig == 0] <- NaN
acc_hist_sig <- as.integer(acc_hist$p.val[1, , ] < 0.05)
acc_hist_sig[acc_hist_sig == 0] <- NaN
acc_hist_full_sig <- as.integer(acc_hist_full$p.val[1, , ] < 0.05)
acc_hist_full_sig[acc_hist_full_sig == 0] <- NaN

acc_data_recon_sig <- as.integer(acc_data_recon$p.val[1, , ] < 0.05)
acc_data_recon_sig[acc_data_recon_sig == 0] <- NaN
acc_hist_recon_sig <- as.integer(acc_hist_recon$p.val[1, , ] < 0.05)
acc_hist_recon_sig[acc_hist_recon_sig == 0] <- NaN
acc_hist_recon_full_sig <- as.integer(acc_hist_recon_full$p.val[1, , ] < 0.05)
acc_hist_recon_full_sig[acc_hist_recon_full_sig == 0] <- NaN


# ACC difference significance: Whether the difference in correlation between data and hist is significant

## 1) ACC of the PRED and HIST (to take auto-correlation into account.)
ohc_acc_mod <- Corr(data_ss_anom_ens, hist_ss_anom_ens, time_dim = 'sdate', dat_dim = 'dat', 
                    method = 'pearson', pval = TRUE, conf = F)

## 2) Effective sample size
eff_sample <- Eno(obs_ss_anom, time_dim = 'sdate')[1, , ]

## 3) Compute significance with paired.r function
sig_corr <- psych::paired.r(acc_data$corr[1, , ], acc_hist$corr[1, , ], ohc_acc_mod$corr[1, , ], round(eff_sample), twotailed = TRUE)$p < 0.05
#change T/F to 1/0
sig_corr <- sig_corr * 1
sig_corr[sig_corr == 0] <- NaN

sig_corr_recon <- psych::paired.r(acc_data_recon$corr[1, , ], acc_hist_recon$corr[1, , ], ohc_acc_mod$corr[1, , ], round(eff_sample), twotailed = TRUE)$p < 0.05
#change T/F to 1/0
sig_corr_recon <- sig_corr_recon * 1
sig_corr_recon[sig_corr_recon == 0] <- NaN

if (INFO) print("==========STEP7 FINISHED.===========")

#===== Plotting: type II =====
if (savefile) {
  png(paste0(savefile_path, '/dcpp_ts_figII_', exp, '_', vari, '_', reg, '.png'),
      width = 480*1.8, height = 480*0.8)
} else {
  dev.new(width = 9, height = 5)
}

pcexlab <- 1.3
pcexmain <- 1.3
pcexax <- 1.3

# global mean SST, FY1 (Anomaly timeseries (type II))
if (reg %in% c('Global_Ocean')) {
yrs <- seq(1, t2[length(t2)] - t2[1] + 1, by = 1) + as.numeric(substr(sdates[1], 2, 5))  #start from 1961
yrs <- 1961:2021
} else {
yrs <- seq(1, t2[length(t2)] - t2[1] + 1, by = 1) + as.numeric(substr(sdates[1], 2, 5)) - 1 #start from 1960
yrs <- 1960:2020
}

lt <- 1
max_range <- range(range(data_ss_anom_ens[1, , lt], na.rm = T), 
                   range(recon_ss_anom_ens[1, , lt], na.rm = T), 
                   range(hist_ss_anom_ens[1, , lt], na.rm = T), 
                   range(obs_ss_anom_ens[1, , lt], na.rm = T),
                   range(hist_ss_full_anom_ens[1, , lt], na.rm = T))
enlarge_ratio <- 0.0
ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)

type_sdate <- ifelse(all(diff(t2) == 1), 'l', 'p')

if (all(diff(t2) == 1)) { #continuous sdate
  plot(yrs[t2] + (lt - 1), data_ss_anom_ens[1, , lt], col = "black", type = "l", lwd = 1.5, ylim = ylim, xlim = range(yrs), cex.lab = pcexlab, 
       cex.main = pcexmain, cex.axis = pcexax, xlab = "Start Date", ylab = "T", main = paste0('Anomaly, leadtime ', lt, ', ', vari, ', ', reg), lty = 1)    
  lines(yrs[t2] + (lt - 1), recon_ss_anom_ens[1, , lt], col = "grey", lwd = 1.5)
  lines(yrs[t2] + (lt - 1), hist_ss_anom_ens[1, , lt], col = "blue", lwd = 1.5)
  lines(yrs[t2] + (lt - 1), obs_ss_anom_ens[1, , lt], col = "red", lwd = 1.5)
  lines(yrs + (lt - 1), hist_ss_full_anom_ens[1, , lt], col = "blue", lwd = 1.5, lty = 2)

} else { #discrete sdate
  plot(yrs + (lt - 1), y = NULL, col = "black", type = type_sdate, lwd = 0.5, ylim = ylim, xlim = range(yrs), cex.lab = pcexlab, 
       cex.main = pcexmain, cex.axis = pcexax, xlab = "Start Date", ylab = "T", main = paste0('Anomaly, leadtime ', lt, ', ', vari, ', ', reg), lty = 1)

  lines(yrs[t2] + (lt - 1), data_ss_anom_ens[1, , lt], col = 'black', type = type_sdate, lwd = 1.5)
  lines(yrs[t2] + (lt - 1), recon_ss_anom_ens[1, , lt], col = 'grey', type = type_sdate, lwd = 1.5)
  lines(yrs[t2] + (lt - 1), hist_ss_anom_ens[1, , lt], col = 'blue', type = type_sdate, lwd = 1.5)
  lines(yrs[t2] + (lt - 1), obs_ss_anom_ens[1, , lt], col = 'red', type = type_sdate, lwd = 1.5)
  lines(yrs + (lt - 1), hist_ss_full_anom_ens[1, , lt], col = 'blue', type = 'l',, lwd = 1.5, lty = 2)
#  for (t in seq(1, length(t2))) {
#    lines(yrs[t2[t] - t2[1] + 1] + (lt - 1), data_ss_anom_ens[1, t, lt], col = 'black', type = type_sdate, lwd = 1.5) 
#    lines(yrs[t2[t] - t2[1] + 1] + (lt - 1), recon_ss_anom_ens[1, t, lt], col = 'grey', type = type_sdate, lwd = 1.5) 
#    lines(yrs[t2[t] - t2[1] + 1] + (lt - 1), hist_ss_anom_ens[1, t, lt], col = 'blue', type = type_sdate, lwd = 1.5) 
#    lines(yrs[t2[t] - t2[1] + 1] + (lt - 1), obs_ss_anom_ens[1, t, lt], col = 'red', type = type_sdate, lwd = 1.5) 
#  }
#  lines(yrs + (lt - 1), hist_full_ss_anom_ens[1, , lt], col = 'blue', type = 'l',, lwd = 1.5, lty = 2)
}

# Decide legend position
legend_pos <- ifelse(data_ss_anom_ens[1, 1, lt] > mean(ylim), "bottomleft", "topleft") 
legend(legend_pos, 
       legend = c(exp, recon, 'Hist', obs, 'Hist_full'), 
       col = c("black", 'grey', "blue", "red", "blue"), 
       lty = c(1, 1, 1, 1, 2), bty = "n")

if (savefile) dev.off()

if (INFO) print("==========PLOTTING TYPE II FINISHED.===========")

#===== Plotting: type III (Lead-time dependent climatology) =====
if (savefile) {
  png(paste0(savefile_path, '/dcpp_ts_figIII_', exp, '_', vari, '_', reg, '.png'),
      width = 480*1.8, height = 480*0.8)
} else {
  dev.new(width = 9, height = 5)
}

#NOTE: fyr is different; global_mean is 1:60 and nino3.4 is 1:30
fyr <- c(1:dim(clim_data$clim_exp)['time'])
if (reg == 'Nino3.4') {
    xlabel <- 'Forecast Month'
    } else {
    xlabel <- 'Forecast Year'
}

max_range <- range(range(clim_data$clim_exp[1, ]), 
                   range(clim_recon$clim_exp[1, ]), 
                   range(clim_hist$clim_exp[1, ]), 
                   range(clim_data$clim_obs[1, ]),
                   range(clim_hist_full$clim_exp[1, ]))
enlarge_ratio <- 0.2
ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)

plot(fyr, clim_data$clim_exp[1, ], col = "black", type = "l", lwd = 1, 
     ylim = ylim, xlim = range(fyr), cex.lab = pcexlab, cex.main = pcexmain, 
     cex.axis = pcexax,
     xlab = xlabel, ylab = "T", main = paste0("Climatology, ", vari, ', ', reg), lty = 1)
lines(fyr, clim_recon$clim_exp[1, ], col = 'grey', type = "l", lwd = 1)
lines(fyr, clim_hist$clim_exp[1, ], col = "blue", type = "l", lwd = 1)
lines(fyr, clim_data$clim_obs[1, ], col = 'red', type = "l", lwd = 1)
lines(fyr, clim_hist_full$clim_exp[1, ], col = 'blue', type = 'l', lty = 2, lwd = 1)

# Decide legend position
legend_y_pos <- ifelse(clim_data$clim_exp[1, 1] > mean(ylim), ylim[1] + (mean(ylim) - ylim[1]) / 1.5, ylim[2])
legend(1, legend_y_pos, 
       legend = c(exp, recon, 'Hist', obs, 'Hist_full'), 
       col = c("black", "grey", "blue", "red", "blue"), 
       lty = c(1, 1, 1, 1, 2), bty = "n")

if (savefile) dev.off()

if (INFO) print("==========PLOTTING TYPE III FINISHED.===========")

#===== Plotting: type IV (ACC) =====
if (savefile) {
  png(paste0(savefile_path, '/dcpp_ts_figIV_', exp, '_', vari, '_', reg, '.png'),
      width = 480*1.8, height = 480)
} else {
  dev.new(width = 8, height = 6)
}
par(mfrow = c(2, 1), cex = 0.65)


#--- figIVa: exp-obs ---
#line exp-obs:
plot(fyr, acc_data$corr[1, 1, ], col = "black", type = "l", lwd = 1, 
     ylim = c(-1, 1), xlim = range(fyr), cex.lab = pcexlab, cex.main = pcexmain, 
     cex.axis = pcexax, xlab = xlabel, ylab = "Cor", 
     main = paste0("a) ACC w/ obs, ", reg, ", ", vari), lty = 1)
#sig1: pch1 = circle
lines(fyr, acc_data$corr[1, 1, ] * acc_data_sig, col = "black", type = "p", lwd = 1.0, pch = 1, cex = 1)
#sig2: pch16 = filled circle
lines(fyr, acc_data$corr[1, 1, ] * sig_corr, col = "black", type = "p", lwd = 1.0, pch = 16, cex = 1)
#line hist-obs:
lines(fyr, acc_hist$corr[1, 1, ], col = 'blue', type = "l", lwd = 1)
lines(fyr, acc_hist$corr[1, 1, ] * acc_hist_sig, col = "blue", type = "p", lwd = 1.0, pch = 1, cex = 1)
#line hist-obs_full:
lines(fyr, acc_hist_full$corr[1, 1, ], col = 'blue', type = "l", lty = 2, lwd = 1)
lines(fyr, acc_hist_full$corr[1, 1, ] * acc_hist_full_sig, col = "blue", type = "p", lwd = 1.0, pch = 1, cex = 1)

# Decide legend position
if (acc_data$corr[1, 1, 1] > -0.3 & acc_hist$corr[1, 1, 1] > -0.3 & acc_hist_full$corr[1, 1, 1] > -0.3) {
    legend_pos <- 'bottomleft' 
} else if (acc_data$corr[1, 1, 1] < 0.3 & acc_hist$corr[1, 1, 1] < 0.3 & acc_hist_full$corr[1, 1, 1] < 0.3) {
    legend_pos <- 'topleft' 
} else {
    legend_pos <- 'left'
}
legend(legend_pos, 
       legend = c(paste0(exp, '-', obs), paste0('Hist-', obs), paste0('Hist-', obs, '_full')), 
       col = c("black", "blue", "blue"), 
       lty = c(1, 1, 2), bty = "n")

#--- figIVb: exp-recon ---
#line exp-recon:
plot(fyr, acc_data_recon$corr[1, 1, ], col = "black", type = "l", lwd = 1, 
     ylim = c(-1, 1), xlim = range(fyr), cex.lab = pcexlab, cex.main = pcexmain,
     cex.axis = pcexax, xlab = xlabel, ylab = "Cor", 
     main = paste0("b) ACC w/ recon, ", reg, ", ", vari), lty = 1)
#sig1: pch1 = circle
lines(fyr, acc_data_recon$corr[1, 1, ] * acc_data_sig, col = "black", 
      type = "p", lwd = 1.0, pch = 1, cex = 1)
#sig2: pch16 = filled circle
lines(fyr, acc_data_recon$corr[1, 1, ] * sig_corr, col = "black", 
      type = "p", lwd = 1.0, pch = 16, cex = 1)
#line hist-recon:
lines(fyr, acc_hist_recon$corr[1, 1, ], col = 'blue', type = "l", lwd = 1)
lines(fyr, acc_hist_recon$corr[1, 1, ] * acc_hist_recon_sig, col = "blue",
      type = "p", lwd = 1.0, pch = 1, cex = 1)
#line hist-recon_full:
lines(fyr, acc_hist_recon_full$corr[1, 1, ], col = 'blue', type = "l", lty = 2, lwd = 1)
lines(fyr, acc_hist_recon_full$corr[1, 1, ] * acc_hist_recon_full_sig, col = "blue",
      type = "p", lwd = 1.0, pch = 1, cex = 1)


# Decide legend position
if (acc_data_recon$corr[1, 1, 1] > -0.3 & acc_hist_recon$corr[1, 1, 1] > -0.3 & acc_hist_recon_full$corr[1, 1, 1] > -0.3) {
    legend_pos <- 'bottomleft'
} else if (acc_data_recon$corr[1, 1, 1] < 0.3 & acc_hist_recon$corr[1, 1, 1] < 0.3 & acc_hist_recon_full$corr[1, 1, 1] < 0.3) {
    legend_pos <- 'topleft'
} else {
    legend_pos <- 'left'
}
legend(legend_pos, 
       legend = c(paste0(exp, '-', recon), paste0('Hist-', recon), paste0('Hist-', recon, '_full')), 
       col = c("black", "blue", "blue"), 
       lty = c(1, 1, 2), bty = "n")

if (savefile) dev.off()

if (INFO) print("==========PLOTTING TYPE IV FINISHED.===========")

if (INFO) print("==========FINISHED SUCCESSFULLY.================")

if (Write_to_log) sink()
