fun_Season_diffClim_ACC <- function(data, data_obs) {
  # data: [sdate, time, memb]
  # data_obs: [sdate, time]
  # chosen_mon: c(moninf, monsup)
# For bias: 1st month, 1st seasons, 1st year
# For acc: 1st seasons, 1st year, 2-5yrs, 6-10yrs
#--> The output will be a list of 2 (bias and acc), bias has 6 lists and acc has 7lists, each has a number [time]

  #NOTE: The data should start at November. Season() takes 11 as monini.

  res_bias <- vector('list', 6)
  names(res_bias) <- c('Mon1', 'DJF', 'MAM', 'JJA', 'SON', 'Yr1')
  res_acc <- vector('list', 7)
  names(res_acc) <- c('DJF', 'MAM', 'JJA', 'SON', 'Yr1', 'Yr2-5', 'Yr6-10')
  res_acc_sig <- vector('list', 7)
  names(res_acc_sig) <- c('DJF', 'MAM', 'JJA', 'SON', 'Yr1', 'Yr2-5', 'Yr6-10')

  monini <- 11
  chosen_mon <- list(Mon1 = c(11, 11), # only for bias
                     DJF = c(12, 2),
                     MAM = c(3, 5),
                     JJA = c(6, 8),
                     SON = c(9, 11),
                     Annual = c(1, 12))


  for (season_i in 1:length(chosen_mon)) {

    #================================
    # Step 1: Season()
    #================================
  
    moninf <- chosen_mon[[season_i]][1]
    monsup <- chosen_mon[[season_i]][2]
  
    # Correction need if monini is not January: 
    moninf <- moninf - monini + 1
    monsup <- monsup - monini + 1
    moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
    monsup <- ifelse(monsup <= 0, monsup + 12, monsup)
    while (monsup < moninf) {
      monsup <- monsup + 12
    }
  
    data_yrs <- apply(data, c(1, 3), s2dv:::.Season, monini = monini, moninf = moninf, monsup = monsup, method = mean, na.rm = TRUE)
    data_obs_yrs <- apply(data_obs, 1, s2dv:::.Season, monini = monini, moninf = moninf, monsup = monsup, method = mean, na.rm = TRUE)
    #data_yrs should be [time, sdate, memb], data_obs_yrs should be [time, sdate]
  
    if (length(dim(data_yrs)) != 3) {
      data_yrs <- array(data_yrs, dim = c(time = 1, dim(data_yrs)['sdate'], dim(data_yrs)['memb']))
    }
    if (length(dim(data_obs_yrs)) != 2) {
      data_obs_yrs <- array(data_obs_yrs, dim = c(time = 1, sdate = length(data_obs_yrs)))
    }

    # Subset data_yrs and data_obs_yrs:
    # For 1st mon/season/yr, only need time = 1;
    # For 2-5yr and 6-10yr, need to do average.
    if (season_i < 6) {
      data_yrs <- ClimProjDiags::Subset(data_yrs, 1, 1)
      data_obs_yrs <- ClimProjDiags::Subset(data_obs_yrs, 1, 1)
    } else {
      tmp <- array(dim = c(3, dim(data_yrs)[2:3]))
      tmp[1, , ] <- ClimProjDiags::Subset(data_yrs, 1, 1)
      if (dim(data_yrs)[1] >= 5) {
        tmp[2, , ] <- s2dv::MeanDims(ClimProjDiags::Subset(data_yrs, 1, 2:5), 1) #Yr2-5
      }
      if (dim(data_yrs)[1] >= 10) {
        tmp[3, , ] <- s2dv::MeanDims(ClimProjDiags::Subset(data_yrs, 1, 6:10), 1) #Yr6-10
      }
      data_yrs <- tmp

      tmp <- array(dim = c(3, dim(data_obs_yrs)[2]))
      tmp[1, ] <- ClimProjDiags::Subset(data_obs_yrs, 1, 1)
      if (dim(data_obs_yrs)[1] >= 5) {
        tmp[2, ] <- s2dv::MeanDims(ClimProjDiags::Subset(data_obs_yrs, 1, 2:5), 1) #Yr2-5
      }
      if (dim(data_obs_yrs)[1] >= 10) {
        tmp[3, ] <- s2dv::MeanDims(ClimProjDiags::Subset(data_obs_yrs, 1, 6:10), 1) #Yr6-10
      }
      data_obs_yrs <- tmp
      #Now, data_yrs and data_obs_yrs should be [3, sdate, (memb)], 3 represents Yr1, Yr2-5, and Yr6-10
    }
  
  #    ====CHECK====
  #    if (any(names(dim(data_yrs)) != c('time', 'sdate', 'memb')) |
  #        any(names(dim(data_obs_yrs)) != c('time', 'sdate'))) {
  #      stop("fun_Season_diffClim_ACC error 1.")
  #    }
  
    #================================
    # Step 2: Clim()
    #================================
    clim_data <- array(dim = c(dim(data_yrs)[1]))
    clim_data_obs <- array(dim = c(dim(data_yrs)[1]))
    for (tt in 1:dim(data_yrs)[1]) { #loop through time dim
      clim_data[tt] <- mean(data_yrs[tt, , ])
      clim_data_obs[tt] <- mean(data_obs_yrs[tt, ])
    }
  
    # result 1: diff (clim - obs)
    diff <- clim_data - clim_data_obs
    # diff should be a vector of length of time_dim
  
    res_bias[[season_i]] <- diff[1] # because only need 1st mon/season/yr
  
  #-----------------------------------------------------------------
    if (season_i != 1) {
  
      # Start calculating ACC....
      #=======================================
      # Step 3: Ano()
      #=======================================
      data_yrs_list <- plyr::alply(data_yrs, 2:3)
      data_yrs_anom <- array(unlist(lapply(data_yrs_list, '-', clim_data)),
                             dim = dim(data_yrs))
      data_obs_yrs_list <- plyr::alply(data_obs_yrs, 2)
      data_obs_yrs_anom <- array(unlist(lapply(data_obs_yrs_list, '-', clim_data_obs)),
                                 dim = dim(data_obs_yrs))
#    print('CHECK anom:')
#    print(dim(data_yrs_anom))
#    print(dim(data_obs_yrs_anom))
    
      #=======================================
      # Step 4: Ensemble mean
      #=======================================
      data_yrs_anom_ens <- apply(data_yrs_anom, 1:2, mean)
      if (length(dim(data_yrs_anom_ens)) != 2) {
        data_yrs_anom_ens <- array(data_yrs_anom_ens, dim = c(time = 1, dim(data_yrs_anom)['sdate']))
      }
      #data_yrs_anom_ens and data_obs_yrs_anom should be [time, sdate]
    
#    print('CHECK anom_ens:')
#    print(dim(data_yrs_anom_ens))
    
      #=======================================
      # Step 5: correlation
      #=======================================
      corr <- array(dim = dim(data_yrs_anom_ens)[1]) #time dim
      eno <- array(dim = dim(data_yrs_anom_ens)[1])
      for (i_time in 1:dim(data_yrs_anom_ens)[1]) {
        corr[i_time] <- cor(data_yrs_anom_ens[i_time, ], data_obs_yrs_anom[i_time, ],
                            use = "pairwise.complete.obs",
                            method = 'pearson')
        eno[i_time] <- s2dv:::.Eno(data_obs_yrs_anom[i_time, ], na.action = na.pass)
      }
      # corr and eno should be [time]

      #=======================================
      # Step 6: ACC sig
      #=======================================
      t <- sqrt(corr * corr * (eno - 2) / (1 - (corr ^ 2)))
      p.val <- pt(t, eno - 2, lower.tail = FALSE)
      # p.val should be [time]
      acc_data_sig <- as.integer(p.val < 0.05)
      acc_data_sig[acc_data_sig == 0] <- NaN
      dim(acc_data_sig) <- c(time = length(acc_data_sig))
    
      if (names(chosen_mon)[season_i] != 'Annual') { # Seasons 
        res_acc[[names(chosen_mon)[season_i]]] <- corr[1]  #only first season needed
        res_acc_sig[[names(chosen_mon)[season_i]]] <- acc_data_sig[1]
      } else {
        res_acc[['Yr1']] <- corr[1]
        res_acc_sig[['Yr1']] <- acc_data_sig[1]
        res_acc[['Yr2-5']] <- corr[2]
        res_acc_sig[['Yr2-5']] <- acc_data_sig[2]
        res_acc[['Yr6-10']] <- corr[3]
        res_acc_sig[['Yr6-10']] <- acc_data_sig[3]
      }
  
    } # if season_i != 1
  } # for loop

  # Create the output list:
  # each list is a vector with name 'time'
  res_bias <- array(unlist(res_bias), dim = c(time = length(res_bias)))
  res_acc <- array(unlist(res_acc), dim = c(time = length(res_acc)))
  res_acc_sig <- array(unlist(res_acc_sig), dim = c(time = length(res_acc_sig)))

#  return(list(diff = diff, acc = corr, acc_sig = acc_data_sig))
  return(list(res_bias = res_bias, res_acc = res_acc, res_acc_sig = res_acc_sig))
  }
